"""HTTP client for the Relayway API."""
import json
from typing import Any, Dict

import requests

from .config import API_KEY, BASE_URL


class RelayWayError(RuntimeError):
    """Raised when the Relayway API returns an error."""


def enqueue_job(payload: Dict[str, Any], origin: str) -> Dict[str, Any]:
    """Enqueue a background job to the Relayway API."""
    task_data = {
        "payload": payload,
        "origin": origin,
        "language": "python",
        "status": "pending",
    }

    try:
        json.dumps(task_data)
    except (TypeError, ValueError) as exc:
        raise ValueError("Job payload must be JSON serializable") from exc

    headers = {"Content-Type": "application/json"}
    if API_KEY:
        headers["X-API-Key"] = API_KEY

    response = requests.post(
        f"{BASE_URL.rstrip('/')}/api/v1/tasks/create-task",
        json=task_data,
        headers=headers,
        timeout=10
    )

    if response.status_code >= 400:
        raise RelayWayError(f"Relayway API error: {response.status_code} {response.text}")

    return response.json()



"""Tests for configuration loading."""
import importlib
import os


def test_config_reads_api_key_from_environment(monkeypatch):
    monkeypatch.setenv("RELAYWAY_API_KEY", "sk_test_123")
    import relayway.config as config
    reloaded = importlib.reload(config)
    assert reloaded.API_KEY == "sk_test_123"


def test_config_reads_webhook_secret_from_environment(monkeypatch):
    monkeypatch.setenv("RELAYWAY_WEBHOOK_SECRET", "whsec_test_456")
    import relayway.config as config
    reloaded = importlib.reload(config)
    assert reloaded.WEBHOOK_SECRET == "whsec_test_456"


def test_config_reads_base_url_from_environment(monkeypatch):
    monkeypatch.setenv("RELAYWAY_BASE_URL", "https://api.relayway.com")
    import relayway.config as config
    reloaded = importlib.reload(config)
    assert reloaded.BASE_URL == "https://api.relayway.com"


def test_config_base_url_defaults_when_not_set(monkeypatch):
    monkeypatch.delenv("RELAYWAY_BASE_URL", raising=False)
    import relayway.config as config
    reloaded = importlib.reload(config)
    assert reloaded.BASE_URL == "http://localhost:8000"


def test_config_api_key_is_none_when_not_set(monkeypatch):
    monkeypatch.delenv("RELAYWAY_API_KEY", raising=False)
    import relayway.config as config
    reloaded = importlib.reload(config)
    assert reloaded.API_KEY is None


def test_config_webhook_secret_is_none_when_not_set(monkeypatch):
    monkeypatch.delenv("RELAYWAY_WEBHOOK_SECRET", raising=False)
    import relayway.config as config
    reloaded = importlib.reload(config)
    assert reloaded.WEBHOOK_SECRET is None



"""Tests for the Relayway API client."""
from unittest.mock import Mock

import pytest

from relayway.client import RelayWayError, enqueue_job


def test_enqueue_job_posts_expected_payload(monkeypatch):
    response = Mock()
    response.status_code = 201
    response.json.return_value = {"id": "task-123", "status": "pending"}

    post_mock = Mock(return_value=response)

    monkeypatch.setattr("relayway.client.BASE_URL", "http://api.relayway.com")
    monkeypatch.setattr("relayway.client.API_KEY", "sk_test_key")
    monkeypatch.setattr("relayway.client.requests.post", post_mock)

    result = enqueue_job(
        payload={"to": "test@example.com"},
        origin="http://customer.com/webhook"
    )

    assert result == {"id": "task-123", "status": "pending"}
    post_mock.assert_called_once_with(
        "http://api.relayway.com/api/v1/tasks/create-task",
        json={
            "payload": {"to": "test@example.com"},
            "origin": "http://customer.com/webhook",
            "language": "python",
            "status": "pending",
        },
        headers={"Content-Type": "application/json", "X-API-Key": "sk_test_key"},
        timeout=10,
    )


def test_enqueue_job_works_without_api_key(monkeypatch):
    response = Mock()
    response.status_code = 201
    response.json.return_value = {"id": "task-123"}

    post_mock = Mock(return_value=response)

    monkeypatch.setattr("relayway.client.BASE_URL", "http://localhost:8000")
    monkeypatch.setattr("relayway.client.API_KEY", None)
    monkeypatch.setattr("relayway.client.requests.post", post_mock)

    enqueue_job(payload={}, origin="http://localhost/webhook")

    call_args = post_mock.call_args
    assert "X-API-Key" not in call_args.kwargs["headers"]


def test_enqueue_job_raises_on_non_serializable_payload(monkeypatch):
    monkeypatch.setattr("relayway.client.BASE_URL", "http://localhost:8000")
    monkeypatch.setattr("relayway.client.API_KEY", None)

    with pytest.raises(ValueError, match="JSON serializable"):
        enqueue_job(payload={"bad": object()}, origin="http://localhost/webhook")


def test_enqueue_job_raises_relayway_error_for_http_errors(monkeypatch):
    response = Mock()
    response.status_code = 500
    response.text = "Internal Server Error"

    monkeypatch.setattr("relayway.client.BASE_URL", "http://localhost:8000")
    monkeypatch.setattr("relayway.client.API_KEY", None)
    monkeypatch.setattr("relayway.client.requests.post", Mock(return_value=response))

    with pytest.raises(RelayWayError, match="500"):
        enqueue_job(payload={}, origin="http://localhost/webhook")
